'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var PropTypes = _interopDefault(require('prop-types'));

var initialState = {
  sensitivity: 0.5,
  zoomInStep: 3,
  zoomOutStep: 2,
  dbClickStep: 5,
  pinchSensitivity: 1,
  positionX: 0,
  positionY: 0,
  scale: 1,
  maxScale: 8,
  minScale: 0.5,
  wheelAnimationSpeed: 0,
  zoomAnimationSpeed: 200,
  pinchAnimationSpeed: 0,
  panAnimationSpeed: 0,
  resetAnimationSpeed: 0,
  maxPositionX: null,
  minPositionX: null,
  maxPositionY: null,
  minPositionY: null,
  limitToBounds: true,
  disabled: false,
  zoomingEnabled: true,
  panningEnabled: true,
  pinchEnabled: true,
  dbClickEnabled: true,
  transformEnabled: true,
  enableZoomedOutPanning: false,
  wrapperComponent: null,
  contentComponent: null,
  startPinchDistance: null,
  startPanningCoords: null,
  isDown: null,
  lastPositionZoomEnabled: false,
  enableZoomThrottling: false,
  distance: null,
  previousDistance: null
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

/**
 * Rounds number to given decimal
 * eg. roundNumber(2.34343, 1) => 2.3
 */
var roundNumber = function roundNumber(num) {
  var decimal = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 5;

  return Number(num.toFixed(decimal));
};

/**
 * Checks if value is number, if not it returns default value
 * 1# eg. checkIsNumber(2, 30) => 2
 * 2# eg. checkIsNumber(null, 30) => 30
 */
var checkIsNumber = function checkIsNumber(num, defaultValue) {
  return typeof num === "number" ? num : defaultValue;
};

/**
 * Keeps value between given bounds, used for limiting view to given boundaries
 * 1# eg. boundLimiter(2, 0, 3, true) => 2
 * 2# eg. boundLimiter(4, 0, 3, true) => 3
 * 3# eg. boundLimiter(-2, 0, 3, true) => 0
 * 4# eg. boundLimiter(10, 0, 3, false) => 10
 */
var boundLimiter = function boundLimiter(value, minBound, maxBound, isActive) {
  if (!isActive) return roundNumber(value, 2);
  if (value < minBound) return roundNumber(minBound, 2);
  if (value > maxBound) return roundNumber(maxBound, 2);
  return roundNumber(value, 2);
};

/**
 * Returns relative coords of mouse on wrapper element, and provides
 * info about it's width, height, with same info about its content(zoomed component) element
 */
var relativeCoords = function relativeCoords(event, wrapperComponent, contentComponent, panningCase) {
  var wrapperWidth = wrapperComponent.offsetWidth;
  var wrapperHeight = wrapperComponent.offsetHeight;
  var contentRect = contentComponent.getBoundingClientRect();
  var contentWidth = contentRect.width;
  var contentHeight = contentRect.height;
  var contentLeft = contentRect.left;
  var contentRight = contentRect.right;
  var diffHeight = wrapperHeight - contentHeight;
  var diffWidth = wrapperWidth - contentWidth;

  // mouse position x, y over wrapper component
  var x = panningCase ? event.clientX : event.clientX - contentRect.left;
  var y = panningCase ? event.clientY : event.clientY - contentRect.top;

  // Mobile touch event case
  if (isNaN(x) && event.touches && event.touches[0]) {
    x = event.touches[0].clientX;
    y = event.touches[0].clientY;
  }

  return {
    x: x,
    y: y,
    wrapperWidth: wrapperWidth,
    wrapperHeight: wrapperHeight,
    contentWidth: contentWidth,
    contentHeight: contentHeight,
    diffHeight: diffHeight,
    diffWidth: diffWidth,
    contentLeft: contentLeft,
    contentRight: contentRight
  };
};

/**
 * Calculate bounding area of zoomed/panned element
 */
var calculateBoundingArea = function calculateBoundingArea(wrapperWidth, contentWidth, diffWidth, wrapperHeight, contentHeight, diffHeight, enableZoomedOutPanning) {
  var scaleWidthFactor = wrapperWidth > contentWidth ? diffWidth * (enableZoomedOutPanning ? 1 : 0.5) : 0;
  var scaleHeightFactor = wrapperHeight > contentHeight ? diffHeight * (enableZoomedOutPanning ? 1 : 0.5) : 0;
  var minPositionX = wrapperWidth - contentWidth - scaleWidthFactor;
  var maxPositionX = 0 + scaleWidthFactor;
  var minPositionY = wrapperHeight - contentHeight - scaleHeightFactor;
  var maxPositionY = 0 + scaleHeightFactor;

  return { minPositionX: minPositionX, maxPositionX: maxPositionX, minPositionY: minPositionY, maxPositionY: maxPositionY };
};

/**
 * Returns middle coordinates x,y of two points
 * Used to get middle point of two fingers pinch
 */

var getMiddleCoords = function getMiddleCoords(firstPoint, secondPoint, contentComponent, scale) {
  var contentRect = contentComponent.getBoundingClientRect();

  return {
    x: ((firstPoint.clientX + secondPoint.clientX) / 2 - contentRect.left) / scale,
    y: ((firstPoint.clientY + secondPoint.clientY) / 2 - contentRect.top) / scale
  };
};

/**
 * Returns distance between two points x,y
 */
var getDistance = function getDistance(firstPoint, secondPoint) {
  return Math.hypot(firstPoint.pageX - secondPoint.pageX, firstPoint.pageY - secondPoint.pageY);
};

/**
 * Delete undefined values from object keys
 * Used for deleting empty props
 */

var deleteUndefinedProps = function deleteUndefinedProps(value) {
  var newObject = _extends({}, value);
  Object.keys(newObject).forEach(function (key) {
    return newObject[key] == undefined && delete newObject[key];
  });
  return newObject;
};

/**
 * Returns center zoom position, for computations, based on the last mouse position
 */

var getLastPositionZoomCoords = function getLastPositionZoomCoords(_ref) {
  var lastPositionZoomEnabled = _ref.lastPositionZoomEnabled,
      lastMouseEventPosition = _ref.lastMouseEventPosition,
      previousScale = _ref.previousScale,
      scale = _ref.scale,
      wrapperComponent = _ref.wrapperComponent,
      contentComponent = _ref.contentComponent,
      positionX = _ref.positionX,
      positionY = _ref.positionY,
      resetLastMousePosition = _ref.resetLastMousePosition;

  if (lastPositionZoomEnabled) {
    if (lastMouseEventPosition) {
      if (previousScale === 1 || previousScale >= 1 && scale <= 1 || previousScale <= 1 && scale >= 1) {
        resetLastMousePosition();
        return true;
      }
      return lastMouseEventPosition;
    }
  } else {
    return getRelativeZoomCoords({
      scale: scale,
      wrapperComponent: wrapperComponent,
      contentComponent: contentComponent,
      positionX: positionX,
      positionY: positionY
    });
  }
};

/**
 * Returns center zoom position, for computations, based on the relative center to content node
 */

var getRelativeZoomCoords = function getRelativeZoomCoords(_ref2) {
  var wrapperComponent = _ref2.wrapperComponent,
      contentComponent = _ref2.contentComponent,
      scale = _ref2.scale,
      positionX = _ref2.positionX,
      positionY = _ref2.positionY;

  var _relativeCoords = relativeCoords(event, wrapperComponent, contentComponent, true),
      wrapperWidth = _relativeCoords.wrapperWidth,
      wrapperHeight = _relativeCoords.wrapperHeight;

  var x = (Math.abs(positionX) + wrapperWidth / 2) / scale;
  var y = (Math.abs(positionY) + wrapperHeight / 2) / scale;
  return { x: x, y: y };
};

/**
 * Fire callback if it's function
 */

var handleCallback = function handleCallback(callback, props) {
  if (callback && typeof callback === "function") {
    callback(props);
  }
};

// We want to make event listeners non-passive, and to do so have to check

function makePassiveEventOption(passive) {
  return passive;
}

var Context = React__default.createContext({});

var reset = null;
var timer = null;
var timerTime = 100;
var throttle = false;
var throttleTime = 1;

var distance = null;
var previousDistance = null;

var StateProvider = function (_Component) {
  inherits(StateProvider, _Component);

  function StateProvider() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, StateProvider);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = StateProvider.__proto__ || Object.getPrototypeOf(StateProvider)).call.apply(_ref, [this].concat(args))), _this), _this.state = _extends({}, initialState, _this.props.defaultValues, {
      lastMouseEventPosition: null,
      previousScale: initialState.scale,
      eventType: false
    }), _this.handleZoom = function (event, customMousePosition, customDelta, customStep, customScale, coords) {
      var _this$state = _this.state,
          isDown = _this$state.isDown,
          zoomingEnabled = _this$state.zoomingEnabled,
          disabled = _this$state.disabled,
          wrapperComponent = _this$state.wrapperComponent,
          contentComponent = _this$state.contentComponent,
          positionX = _this$state.positionX,
          positionY = _this$state.positionY,
          scale = _this$state.scale,
          enableZoomedOutPanning = _this$state.enableZoomedOutPanning,
          limitToBounds = _this$state.limitToBounds,
          enableZoomThrottling = _this$state.enableZoomThrottling,
          transformEnabled = _this$state.transformEnabled;

      if (throttle) return;
      if (isDown || !zoomingEnabled || disabled || customScale === scale) return;
      event.preventDefault();
      event.stopPropagation();

      var _ref2 = coords || relativeCoords(event, wrapperComponent, contentComponent),
          x = _ref2.x,
          y = _ref2.y,
          wrapperWidth = _ref2.wrapperWidth,
          wrapperHeight = _ref2.wrapperHeight;

      // Calculate new zoom


      var newScale = _this.handleCalculateZoom(customScale, customDelta, customStep);
      if (_this.checkZoomBounds(newScale)) return;

      var scaleDifference = newScale - scale;

      // Mouse position
      var mouseX = checkIsNumber(customMousePosition && customMousePosition.x, x / scale);
      var mouseY = checkIsNumber(customMousePosition && customMousePosition.y, y / scale);

      if (isNaN(mouseX) || isNaN(mouseY)) return console.warn("No mouse or touch offset found");

      // Bounding area details
      var newContentWidth = wrapperWidth * newScale;
      var newContentHeight = wrapperHeight * newScale;

      var newDiffWidth = wrapperWidth - newContentWidth;
      var newDiffHeight = wrapperHeight - newContentHeight;

      // Calculate bounding area

      var _calculateBoundingAre = calculateBoundingArea(wrapperWidth, newContentWidth, newDiffWidth, wrapperHeight, newContentHeight, newDiffHeight, enableZoomedOutPanning),
          minPositionX = _calculateBoundingAre.minPositionX,
          maxPositionX = _calculateBoundingAre.maxPositionX,
          minPositionY = _calculateBoundingAre.minPositionY,
          maxPositionY = _calculateBoundingAre.maxPositionY;

      if (!transformEnabled) return;
      // Calculate new positions
      var newPositionX = -(mouseX * scaleDifference) + positionX;
      var newPositionY = -(mouseY * scaleDifference) + positionY;

      _this.setZoomTransform(newScale, boundLimiter(newPositionX, minPositionX, maxPositionX, limitToBounds), boundLimiter(newPositionY, minPositionY, maxPositionY, limitToBounds), scale, { x: mouseX, y: mouseY });

      // throttle
      if (enableZoomThrottling) {
        throttle = true;
        setTimeout(function () {
          throttle = false;
        }, throttleTime);
      }
    }, _this.handleCalculateZoom = function (customScale, customDelta, customStep) {
      var _this$state2 = _this.state,
          scale = _this$state2.scale,
          sensitivity = _this$state2.sensitivity,
          maxScale = _this$state2.maxScale,
          minScale = _this$state2.minScale;

      if (typeof customScale === "number") return customScale;
      var deltaY = event ? event.deltaY < 0 ? 1 : -1 : 0;
      var delta = checkIsNumber(customDelta, deltaY);
      var zoomSensitivity = sensitivity * 0.1;

      var newScale = typeof customStep !== "number" ? roundNumber(scale + delta * zoomSensitivity * scale, 2) : roundNumber(scale + customStep * delta * scale * 0.1 / 0.5, 2);
      if (!isNaN(maxScale) && newScale >= maxScale && scale < maxScale) {
        newScale = maxScale;
      }
      if (!isNaN(minScale) && newScale <= minScale && scale > minScale) {
        newScale = minScale;
      }
      return newScale;
    }, _this.checkZoomBounds = function (scale) {
      var _this$state3 = _this.state,
          maxScale = _this$state3.maxScale,
          minScale = _this$state3.minScale;

      return !isNaN(maxScale) && !isNaN(minScale) && scale > maxScale || scale < minScale;
    }, _this.handleWheel = function (event) {
      // Wheel start event
      if (!timer) {
        _this.setState({ eventType: "wheel" });
        handleCallback(_this.props.onWheelStart, _this.getCallbackProps());
      }

      //Wheel event
      _this.handleZoom(event);
      handleCallback(_this.props.onWheel, _this.getCallbackProps());

      // Wheel stop event
      clearTimeout(timer);
      timer = setTimeout(function () {
        handleCallback(_this.props.onWheelStop, _this.getCallbackProps());
        _this.setState(function (p) {
          return { eventType: p.eventType === "wheel" ? false : p.eventType };
        });
        timer = null;
      }, timerTime);
    }, _this.handleStartPanning = function (event) {
      var _this$state4 = _this.state,
          isDown = _this$state4.isDown,
          panningEnabled = _this$state4.panningEnabled,
          disabled = _this$state4.disabled,
          wrapperComponent = _this$state4.wrapperComponent,
          contentComponent = _this$state4.contentComponent,
          positionX = _this$state4.positionX,
          positionY = _this$state4.positionY;
      var target = event.target,
          touches = event.touches;


      if (isDown || !panningEnabled || disabled || !wrapperComponent.contains(target) || touches && touches.length !== 1) return;
      if (touches && touches.length === 1) {
        distance = touches[0].clientX;
      } else {
        var points = relativeCoords(event, wrapperComponent, contentComponent, true);

        var startCoords = {
          x: points.x - positionX,
          y: points.y - positionY
        };

        _this.setPanningData(startCoords, true, "pan");
        handleCallback(_this.props.onPanningStart, _this.getCallbackProps());
      }
    }, _this.handlePanning = function (event) {
      var _this$state5 = _this.state,
          isDown = _this$state5.isDown,
          panningEnabled = _this$state5.panningEnabled,
          disabled = _this$state5.disabled,
          wrapperComponent = _this$state5.wrapperComponent,
          contentComponent = _this$state5.contentComponent,
          startPanningCoords = _this$state5.startPanningCoords,
          enableZoomedOutPanning = _this$state5.enableZoomedOutPanning,
          limitToBounds = _this$state5.limitToBounds,
          positionX = _this$state5.positionX,
          positionY = _this$state5.positionY;

      if (event.touches && event.touches.length === 1 && !isDown && Math.abs(distance - event.touches[0].clientX) > 1) {
        // Mobile panning event because everything is based on touch navigation
        event.preventDefault();
        event.stopPropagation();
        var points = relativeCoords(event, wrapperComponent, contentComponent, true);

        var startCoords = {
          x: points.x - positionX,
          y: points.y - positionY
        };

        _this.setPanningData(startCoords, true, "pan");
        handleCallback(_this.props.onPanningStart, _this.getCallbackProps());
      }
      if (!isDown || !panningEnabled || disabled || event.touches && event.touches.length !== 1 || !startPanningCoords) return;
      event.stopPropagation();
      event.preventDefault();

      var _relativeCoords = relativeCoords(event, wrapperComponent, contentComponent, true),
          x = _relativeCoords.x,
          y = _relativeCoords.y,
          wrapperWidth = _relativeCoords.wrapperWidth,
          wrapperHeight = _relativeCoords.wrapperHeight,
          contentWidth = _relativeCoords.contentWidth,
          contentHeight = _relativeCoords.contentHeight,
          diffWidth = _relativeCoords.diffWidth,
          diffHeight = _relativeCoords.diffHeight;

      var newPositionX = roundNumber(x - startPanningCoords.x, 2);
      var newPositionY = roundNumber(y - startPanningCoords.y, 2);

      // Calculate bounding area

      var _calculateBoundingAre2 = calculateBoundingArea(wrapperWidth, contentWidth, diffWidth, wrapperHeight, contentHeight, diffHeight, enableZoomedOutPanning),
          minPositionX = _calculateBoundingAre2.minPositionX,
          maxPositionX = _calculateBoundingAre2.maxPositionX,
          minPositionY = _calculateBoundingAre2.minPositionY,
          maxPositionY = _calculateBoundingAre2.maxPositionY;

      _this.setPositionX(boundLimiter(newPositionX, minPositionX, maxPositionX, limitToBounds));
      _this.setPositionY(boundLimiter(newPositionY, minPositionY, maxPositionY, limitToBounds));
      handleCallback(_this.props.onPanning, _this.getCallbackProps());
    }, _this.handleStopPanning = function () {
      var _this$state6 = _this.state,
          isDown = _this$state6.isDown,
          wrapperComponent = _this$state6.wrapperComponent,
          contentComponent = _this$state6.contentComponent,
          scale = _this$state6.scale,
          positionX = _this$state6.positionX,
          positionY = _this$state6.positionY;

      _this.setIsDown(false);
      if (isDown) {
        distance = null;

        var _getRelativeZoomCoord = getRelativeZoomCoords({
          wrapperComponent: wrapperComponent,
          contentComponent: contentComponent,
          scale: scale,
          positionX: positionX,
          positionY: positionY
        }),
            x = _getRelativeZoomCoord.x,
            y = _getRelativeZoomCoord.y;

        _this.setState(function (p) {
          return {
            lastMouseEventPosition: { x: x, y: y },
            eventType: p.eventType === "pan" ? false : p.eventType
          };
        });
        handleCallback(_this.props.onPanningStop, _this.getCallbackProps());
      }
    }, _this.handlePinchStart = function (event) {
      var disabled = _this.state.disabled;
      var touches = event.touches;

      if (!disabled && touches && touches.length === 1) _this.handleStartPanning(event);
      if (disabled || touches && touches.length !== 2) return;
      event.preventDefault();
      event.stopPropagation();

      _this.setState({
        eventType: "pinch"
      });

      handleCallback(_this.props.onPinchingStart, _this.getCallbackProps());
    }, _this.handlePinch = function (event) {
      var _this$state7 = _this.state,
          pinchSensitivity = _this$state7.pinchSensitivity,
          pinchEnabled = _this$state7.pinchEnabled,
          disabled = _this$state7.disabled,
          wrapperComponent = _this$state7.wrapperComponent,
          contentComponent = _this$state7.contentComponent,
          scale = _this$state7.scale;

      if (event.touches.length === 1) _this.handlePanning(event);
      if (event.touches.length !== 2) return;
      if (pinchEnabled && event.touches.length === 2 && !disabled) {
        var length = getDistance(event.touches[0], event.touches[1]);
        if (typeof distance === "number") {
          if (distance < length && previousDistance > length || distance > length && previousDistance < length) {
            previousDistance = distance;
          }
        } else {
          previousDistance = length;
        }

        distance = length;
        if (typeof previousDistance !== "number") return;
        var diff = previousDistance - distance;
        var delta = diff > 0 ? 1 : -1;

        var _relativeCoords2 = relativeCoords(event, wrapperComponent, contentComponent),
            wrapperWidth = _relativeCoords2.wrapperWidth,
            wrapperHeight = _relativeCoords2.wrapperHeight,
            contentWidth = _relativeCoords2.contentWidth;
        // console.log(previousDistance, distance);


        var factor = (contentWidth - diff) / scale / wrapperWidth * pinchSensitivity;
        var newZoom = roundNumber(scale - factor * scale * delta * 0.025, 2);
        var middleCoords = getMiddleCoords(event.touches[0], event.touches[0], contentComponent, newZoom);

        if (newZoom === scale) return;
        _this.handleZoom(event, middleCoords, null, null, newZoom, _extends({}, middleCoords, {
          wrapperWidth: wrapperWidth,
          wrapperHeight: wrapperHeight,
          contentWidth: contentWidth
        }));
        handleCallback(_this.props.onPinching, _this.getCallbackProps());
      }
    }, _this.handlePinchStop = function () {
      if (typeof distance === "number") {
        _this.setState(function (p) {
          return { eventType: p.eventType === "pinch" ? false : p.eventType };
        });
        previousDistance = null;
        distance = null;
        _this.setState({ middleCoords: null });

        handleCallback(_this.props.onPinchingStop, _this.getCallbackProps());
      }
      _this.handleStopPanning();
    }, _this.resetLastMousePosition = function () {
      return _this.setState({ lastMouseEventPosition: null });
    }, _this.zoomIn = function (event) {
      var _this$state8 = _this.state,
          zoomingEnabled = _this$state8.zoomingEnabled,
          disabled = _this$state8.disabled,
          zoomInStep = _this$state8.zoomInStep,
          scale = _this$state8.scale,
          lastMouseEventPosition = _this$state8.lastMouseEventPosition,
          previousScale = _this$state8.previousScale,
          lastPositionZoomEnabled = _this$state8.lastPositionZoomEnabled,
          wrapperComponent = _this$state8.wrapperComponent,
          contentComponent = _this$state8.contentComponent,
          positionX = _this$state8.positionX,
          positionY = _this$state8.positionY;

      if (!event) return console.error("Zoom in function require event prop");
      if (!zoomingEnabled || disabled) return;
      var zoomCoords = getLastPositionZoomCoords({
        resetLastMousePosition: _this.resetLastMousePosition,
        lastPositionZoomEnabled: lastPositionZoomEnabled,
        lastMouseEventPosition: lastMouseEventPosition,
        previousScale: previousScale,
        scale: scale,
        wrapperComponent: wrapperComponent,
        contentComponent: contentComponent,
        positionX: positionX,
        positionY: positionY
      });
      _this.handleZoom(event, zoomCoords, 1, zoomInStep);
    }, _this.zoomOut = function (event) {
      var _this$state9 = _this.state,
          zoomingEnabled = _this$state9.zoomingEnabled,
          disabled = _this$state9.disabled,
          zoomOutStep = _this$state9.zoomOutStep,
          scale = _this$state9.scale,
          lastMouseEventPosition = _this$state9.lastMouseEventPosition,
          previousScale = _this$state9.previousScale,
          lastPositionZoomEnabled = _this$state9.lastPositionZoomEnabled,
          wrapperComponent = _this$state9.wrapperComponent,
          contentComponent = _this$state9.contentComponent,
          positionX = _this$state9.positionX,
          positionY = _this$state9.positionY;

      if (!event) return console.error("Zoom out function require event prop");
      if (!zoomingEnabled || disabled) return;
      var zoomCoords = getLastPositionZoomCoords({
        resetLastMousePosition: _this.resetLastMousePosition,
        lastPositionZoomEnabled: lastPositionZoomEnabled,
        lastMouseEventPosition: lastMouseEventPosition,
        previousScale: previousScale,
        scale: scale,
        wrapperComponent: wrapperComponent,
        contentComponent: contentComponent,
        positionX: positionX,
        positionY: positionY
      });
      _this.handleZoom(event, zoomCoords, -1, zoomOutStep);
    }, _this.handleDbClick = function (event) {
      var _this$state10 = _this.state,
          zoomingEnabled = _this$state10.zoomingEnabled,
          disabled = _this$state10.disabled,
          dbClickStep = _this$state10.dbClickStep,
          dbClickEnabled = _this$state10.dbClickEnabled;

      if (!event) return console.error("Double click function require event prop");
      if (!zoomingEnabled || disabled || !dbClickEnabled) return;
      _this.handleZoom(event, false, 1, dbClickStep);
    }, _this.setScale = function (scale) {
      _this.setState({ scale: scale });
    }, _this.setPositionX = function (positionX) {
      _this.setState({ positionX: roundNumber(positionX, 3) });
    }, _this.setPositionY = function (positionY) {
      _this.setState({ positionY: roundNumber(positionY, 3) });
    }, _this.setTransform = function (positionX, positionY, scale) {
      if (!_this.state.transformEnabled) return;
      !isNaN(scale) && _this.setScale(scale);
      !isNaN(positionX) && _this.setPositionX(positionX);
      !isNaN(positionY) && _this.setPositionY(positionY);
    }, _this.resetTransform = function (animation) {
      var _this$props$defaultVa = _this.props.defaultValues,
          defaultScale = _this$props$defaultVa.defaultScale,
          defaultPositionX = _this$props$defaultVa.defaultPositionX,
          defaultPositionY = _this$props$defaultVa.defaultPositionY;
      var _this$state11 = _this.state,
          scale = _this$state11.scale,
          positionX = _this$state11.positionX,
          positionY = _this$state11.positionY,
          disabled = _this$state11.disabled;

      if (disabled) return;
      var type = isNaN(animation) ? "reset" : animation;
      _this.setState({ eventType: type });
      if (scale === defaultScale && positionX === defaultPositionX && positionY === defaultPositionY) return;
      _this.setScale(checkIsNumber(defaultScale, initialState.scale));
      _this.setPositionX(checkIsNumber(defaultPositionX, initialState.positionX));
      _this.setPositionY(checkIsNumber(defaultPositionY, initialState.positionY));
      clearTimeout(reset);
      reset = setTimeout(function () {
        _this.setState(function (p) {
          return { eventType: p.eventType === type ? false : p.eventType };
        });
      }, 1);
    }, _this.setZoomTransform = function (scale, positionX, positionY, previousScale, lastMouseEventPosition) {
      _this.setState({ scale: scale, positionX: positionX, positionY: positionY, previousScale: previousScale, lastMouseEventPosition: lastMouseEventPosition });
    }, _this.setPanningData = function (startPanningCoords, isDown, eventType) {
      _this.setState({ startPanningCoords: startPanningCoords, isDown: isDown, eventType: eventType });
    }, _this.setStartPanningCoords = function (startPanningCoords) {
      _this.setState({ startPanningCoords: startPanningCoords });
    }, _this.setStartPinchDistance = function (startPinchDistance) {
      _this.setState({ startPinchDistance: startPinchDistance });
    }, _this.setIsDown = function (isDown) {
      _this.setState({ isDown: isDown });
    }, _this.setDistance = function (distance) {
      _this.setState({ distance: distance });
    }, _this.setPreviousDistance = function (previousDistance) {
      _this.setState({ previousDistance: previousDistance });
    }, _this.setWrapperComponent = function (wrapperComponent) {
      _this.setState({ wrapperComponent: wrapperComponent });
    }, _this.setContentComponent = function (contentComponent) {
      _this.setState({ contentComponent: contentComponent });
    }, _this.getCallbackProps = function () {
      return {
        positionX: _this.state.positionX,
        positionY: _this.state.positionY,
        scale: _this.state.scale,
        sensitivity: _this.state.sensitivity,
        maxScale: _this.state.maxScale,
        minScale: _this.state.minScale,
        wheelAnimationSpeed: _this.state.wheelAnimationSpeed,
        zoomAnimationSpeed: _this.state.zoomAnimationSpeed,
        pinchAnimationSpeed: _this.state.pinchAnimationSpeed,
        panAnimationSpeed: _this.state.panAnimationSpeed,
        minPositionX: _this.state.minPositionX,
        minPositionY: _this.state.minPositionY,
        maxPositionX: _this.state.maxPositionX,
        maxPositionY: _this.state.maxPositionY,
        limitToBounds: _this.state.limitToBounds,
        zoomingEnabled: _this.state.zoomingEnabled,
        panningEnabled: _this.state.panningEnabled,
        transformEnabled: _this.state.transformEnabled,
        pinchEnabled: _this.state.pinchEnabled,
        enableZoomedOutPanning: _this.state.enableZoomedOutPanning,
        disabled: _this.state.disabled,
        zoomOutStep: _this.state.zoomOutStep,
        zoomInStep: _this.state.zoomInStep,
        dbClickStep: _this.state.dbClickStep,
        pinchSensitivity: _this.state.pinchSensitivity,
        dbClickEnabled: _this.state.dbClickEnabled,
        lastPositionZoomEnabled: _this.state.lastPositionZoomEnabled,
        previousScale: _this.state.previousScale
      };
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(StateProvider, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      var _this2 = this;

      var passiveOption = makePassiveEventOption(false);

      // Panning on window to allow panning when mouse is out of wrapper
      window.addEventListener("mousedown", this.handleStartPanning, passiveOption);
      window.addEventListener("mousemove", this.handlePanning, passiveOption);
      window.addEventListener("mouseup", this.handleStopPanning, passiveOption);
      return function () {
        window.removeEventListener("mousedown", _this2.handleStartPanning, passiveOption);
        window.removeEventListener("mousemove", _this2.handlePanning, passiveOption);
        window.removeEventListener("mouseup", _this2.handleStopPanning, passiveOption);
      };
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate(oldProps, oldState) {
      var wrapperComponent = this.state.wrapperComponent;
      var defaultValues = this.props.defaultValues;

      if (!oldState.wrapperComponent && this.state.wrapperComponent) {
        // Zooming events on wrapper
        var passiveOption = makePassiveEventOption(false);
        wrapperComponent.addEventListener("wheel", this.handleWheel, passiveOption);
        wrapperComponent.addEventListener("dblclick", this.handleDbClick, passiveOption);
        wrapperComponent.addEventListener("touchstart", this.handlePinchStart, passiveOption);
        wrapperComponent.addEventListener("touchmove", this.handlePinch, passiveOption);
        wrapperComponent.addEventListener("touchend", this.handlePinchStop, passiveOption);
      }
      if (oldProps.defaultValues !== defaultValues) {
        // eslint-disable-next-line react/no-did-update-set-state
        this.setState(_extends({}, defaultValues));
      }
    }

    //////////
    // Zooming
    //////////

    //////////
    // Wheel
    //////////

    //////////
    // Panning
    //////////

    //////////
    // Pinching
    //////////

    //////////
    // Controls
    //////////

    //////////
    // Setters
    //////////

    // PROPS

  }, {
    key: "render",
    value: function render() {
      /**
       * Context provider value
       */
      var value = {
        state: this.getCallbackProps(),
        dispatch: {
          setScale: this.setScale,
          setPositionX: this.setPositionX,
          setPositionY: this.setPositionY,
          zoomIn: this.zoomIn,
          zoomOut: this.zoomOut,
          setTransform: this.setTransform,
          resetTransform: this.resetTransform
        },
        nodes: {
          setWrapperComponent: this.setWrapperComponent,
          setContentComponent: this.setContentComponent
        },
        internal: {
          handleZoom: this.handleZoom,
          handleStartPanning: this.handleStartPanning,
          handlePanning: this.handlePanning,
          handleStopPanning: this.handleStopPanning,
          handleDbClick: this.handleDbClick,
          handlePinchStart: this.handlePinchStart,
          handlePinch: this.handlePinch,
          handlePinchStop: this.handlePinchStop,
          eventType: this.state.eventType
        }
      };
      var children = this.props.children;

      var content = typeof children === "function" ? children(_extends({}, value.state, value.dispatch)) : children;

      return React__default.createElement(
        Context.Provider,
        { value: value },
        content
      );
    }
  }]);
  return StateProvider;
}(React.Component);

StateProvider.defaultProps = {
  defaultValues: {},
  onWheelStart: null,
  onWheel: null,
  onWheelStop: null,
  onPanningStart: null,
  onPanning: null,
  onPanningStop: null,
  onPinchingStart: null,
  onPinching: null,
  onPinchingStop: null
};

StateProvider.propTypes = {
  children: PropTypes.any,
  defaultValues: PropTypes.object,
  onWheelStart: PropTypes.func,
  onWheel: PropTypes.func,
  onWheelStop: PropTypes.func,
  onPanningStart: PropTypes.func,
  onPanning: PropTypes.func,
  onPanningStop: PropTypes.func,
  onPinchingStart: PropTypes.func,
  onPinching: PropTypes.func,
  onPinchingStop: PropTypes.func
};

var TransformWrapper = function TransformWrapper(_ref) {
  var children = _ref.children,
      scale = _ref.scale,
      positionX = _ref.positionX,
      positionY = _ref.positionY,
      sensitivity = _ref.sensitivity,
      maxScale = _ref.maxScale,
      minScale = _ref.minScale,
      wheelAnimationSpeed = _ref.wheelAnimationSpeed,
      zoomAnimationSpeed = _ref.zoomAnimationSpeed,
      pinchAnimationSpeed = _ref.pinchAnimationSpeed,
      panAnimationSpeed = _ref.panAnimationSpeed,
      resetAnimationSpeed = _ref.resetAnimationSpeed,
      minPositionX = _ref.minPositionX,
      minPositionY = _ref.minPositionY,
      maxPositionX = _ref.maxPositionX,
      maxPositionY = _ref.maxPositionY,
      limitToBounds = _ref.limitToBounds,
      zoomingEnabled = _ref.zoomingEnabled,
      panningEnabled = _ref.panningEnabled,
      transformEnabled = _ref.transformEnabled,
      pinchEnabled = _ref.pinchEnabled,
      enableZoomedOutPanning = _ref.enableZoomedOutPanning,
      disabled = _ref.disabled,
      zoomOutStep = _ref.zoomOutStep,
      zoomInStep = _ref.zoomInStep,
      dbClickStep = _ref.dbClickStep,
      pinchSensitivity = _ref.pinchSensitivity,
      dbClickEnabled = _ref.dbClickEnabled,
      lastPositionZoomEnabled = _ref.lastPositionZoomEnabled,
      enableZoomThrottling = _ref.enableZoomThrottling,
      onWheelStart = _ref.onWheelStart,
      onWheel = _ref.onWheel,
      onWheelStop = _ref.onWheelStop,
      onPanningStart = _ref.onPanningStart,
      onPanning = _ref.onPanning,
      onPanningStop = _ref.onPanningStop,
      onPinchingStart = _ref.onPinchingStart,
      onPinching = _ref.onPinching,
      onPinchingStop = _ref.onPinchingStop;

  return React__default.createElement(
    StateProvider,
    {
      defaultValues: deleteUndefinedProps({
        scale: scale,
        positionX: positionX,
        positionY: positionY,
        sensitivity: sensitivity,
        maxScale: maxScale,
        minScale: minScale,
        wheelAnimationSpeed: wheelAnimationSpeed,
        zoomAnimationSpeed: zoomAnimationSpeed,
        pinchAnimationSpeed: pinchAnimationSpeed,
        panAnimationSpeed: panAnimationSpeed,
        resetAnimationSpeed: resetAnimationSpeed,
        minPositionX: minPositionX,
        minPositionY: minPositionY,
        maxPositionX: maxPositionX,
        maxPositionY: maxPositionY,
        limitToBounds: limitToBounds,
        zoomingEnabled: zoomingEnabled,
        panningEnabled: panningEnabled,
        transformEnabled: transformEnabled,
        pinchEnabled: pinchEnabled,
        enableZoomedOutPanning: enableZoomedOutPanning,
        disabled: disabled,
        zoomOutStep: zoomOutStep,
        zoomInStep: zoomInStep,
        dbClickStep: dbClickStep,
        pinchSensitivity: pinchSensitivity,
        dbClickEnabled: dbClickEnabled,
        lastPositionZoomEnabled: lastPositionZoomEnabled,
        enableZoomThrottling: enableZoomThrottling
      }),
      onWheelStart: onWheelStart,
      onWheel: onWheel,
      onWheelStop: onWheelStop,
      onPanningStart: onPanningStart,
      onPanning: onPanning,
      onPanningStop: onPanningStop,
      onPinchingStart: onPinchingStart,
      onPinching: onPinching,
      onPinchingStop: onPinchingStop
    },
    children
  );
};

TransformWrapper.propTypes = {
  children: PropTypes.any,
  scale: PropTypes.number,
  positionX: PropTypes.number,
  positionY: PropTypes.number,
  sensitivity: PropTypes.number,
  maxScale: PropTypes.oneOfType([PropTypes.number, PropTypes.object]),
  minScale: PropTypes.oneOfType([PropTypes.number, PropTypes.object]),
  wheelAnimationSpeed: PropTypes.number,
  zoomAnimationSpeed: PropTypes.number,
  pinchAnimationSpeed: PropTypes.number,
  panAnimationSpeed: PropTypes.number,
  resetAnimationSpeed: PropTypes.number,
  minPositionX: PropTypes.oneOfType([PropTypes.number, PropTypes.object]),
  maxPositionX: PropTypes.oneOfType([PropTypes.number, PropTypes.object]),
  minPositionY: PropTypes.oneOfType([PropTypes.number, PropTypes.object]),
  maxPositionY: PropTypes.oneOfType([PropTypes.number, PropTypes.object]),
  limitToBounds: PropTypes.bool,
  zoomingEnabled: PropTypes.bool,
  panningEnabled: PropTypes.bool,
  transformEnabled: PropTypes.bool,
  pinchEnabled: PropTypes.bool,
  enableZoomedOutPanning: PropTypes.bool,
  disabled: PropTypes.bool,
  zoomOutStep: PropTypes.bool,
  zoomInStep: PropTypes.bool,
  dbClickStep: PropTypes.bool,
  pinchSensitivity: PropTypes.bool,
  dbClickEnabled: PropTypes.bool,
  lastPositionZoomEnabled: PropTypes.bool,
  enableZoomThrottling: PropTypes.bool,
  onWheelStart: PropTypes.func,
  onWheel: PropTypes.func,
  onWheelStop: PropTypes.func,
  onPanningStart: PropTypes.func,
  onPanning: PropTypes.func,
  onPanningStop: PropTypes.func,
  onPinchingStart: PropTypes.func,
  onPinching: PropTypes.func,
  onPinchingStop: PropTypes.func
};

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css = ".TransformComponent-module_container__3NwNd {\n  position: relative;\n  width: fit-content;\n  height: fit-content;\n  overflow: hidden;\n  -webkit-touch-callout: none; /* iOS Safari */\n  -webkit-user-select: none; /* Safari */\n  -khtml-user-select: none; /* Konqueror HTML */\n  -moz-user-select: none; /* Firefox */\n  -ms-user-select: none; /* Internet Explorer/Edge */\n  user-select: none;\n  margin: 0;\n  padding: 0;\n}\n.TransformComponent-module_content__TZU5O {\n  display: flex;\n  flex-wrap: wrap;\n  width: fit-content;\n  height: fit-content;\n  transform-origin: 0% 0%;\n  margin: 0;\n  padding: 0;\n}\n.TransformComponent-module_content__TZU5O img {\n  pointer-events: none;\n}\n";
var styles = { "container": "TransformComponent-module_container__3NwNd", "content": "TransformComponent-module_content__TZU5O" };
styleInject(css);

function TransformComponent(_ref) {
  var children = _ref.children;

  var wrapperRef = React.useRef(null);
  var contentRef = React.useRef(null);

  var _useContext = React.useContext(Context),
      state = _useContext.state,
      nodes = _useContext.nodes,
      internal = _useContext.internal;

  var style = {
    transform: "translate(" + state.positionX + "px, " + state.positionY + "px) scale(" + state.scale + ")",
    transition: "all " + getAnimationTime(internal.eventType) + "ms ease-in-out"
  };

  function getAnimationTime(type) {
    if (type !== false && !isNaN(type)) return type;
    switch (type) {
      case "pan":
        return state.panAnimationSpeed;
      case "wheel":
        return state.wheelAnimationSpeed;
      case "pinch":
        return state.pinchAnimationSpeed;
      case "reset":
        return state.resetAnimationSpeed;
      default:
        return state.zoomAnimationSpeed;
    }
  }

  React.useEffect(function () {
    if (wrapperRef.current) {
      nodes.setWrapperComponent(wrapperRef.current);
    }
    if (contentRef.current) {
      nodes.setContentComponent(contentRef.current);
    }
  }, []);

  return React__default.createElement(
    "div",
    { ref: wrapperRef, className: "\"react-transform-component\" " + styles.container },
    React__default.createElement(
      "div",
      { ref: contentRef, className: "react-transform-element " + styles.content, style: style },
      children
    )
  );
}

TransformComponent.propTypes = { children: PropTypes.any };

exports.TransformWrapper = TransformWrapper;
exports.TransformComponent = TransformComponent;
//# sourceMappingURL=index.js.map
